@@ -1,15 +1,74 @@
 import pyspark.sql.functions as F
 from pyspark.sql.types import *
+from pyspark.sql import DataFrame
+import datetime
+            
+def cross_sectional(df, event_var, event_id, coDate=None):
+    '''
+    This function makes a cross sectional dataset of the last record before a defined date
+    for records defined in event_id.
+    The date for which the cross sectional dataset is made is defined in the parameter coDate; 
+    if no date is given, the default date is today. 
+    The time at which the event happens is given in the variable event_var.
     
+    :param df: The dataframe of observations from which to make a cross sectional dataset 
+    :type df: dataframe 
+    :param event_var: Name of the column in the dataframe containing the date at which the event happened
+    :type event_var: string 
+    :param event_id: The combination of variables from which the last occurrence on event_var will be returned 
+    :type event_id: list
+    :param coDate: The max date for events on event_var
+    :type coDate: datetime/timestamp
+    
+    Returns: a dataframe
+    Dataframe: A cross sectional dataframe.
+    '''
+    if (isinstance(df, DataFrame)) & (isinstance(event_var, str)) & (isinstance(event_id, list)) & \
+        ((coDate==None) | (isinstance(coDate, datetime.datetime))):
+        if (([dtype for name, dtype in df.dtypes if name == event_var][0])=='timestamp'):
+        
+            if coDate!=None:
+                df = df.filter(F.col(event_var) <= coDate)
+
+            eventlist_ids = event_id.copy()
+            eventlist_ids.append(event_var)
+
+            df_cross = df.join(df.groupBy(event_id).agg(F.max(event_var).alias(event_var)),\
+                                  eventlist_ids, how='inner')
+            return df_cross
+        else:
+            if not (([dtype for name, dtype in df.dtypes if name == event_var][0])=='timestamp'):
+                raise Exception('Variabel event_var i dataframe må ha datatype timestamp')
+                return
+            
+    else:
+        if not (isinstance(df, DataFrame)):
+                raise Exception('Første parameter må være en dataframe.')
+                return
+            
+        if not (isinstance(event_var, str)):
+            raise Exception('Andre parameter må være en string.')
+            return
+
+        if not (isinstance(event_id, list)):
+            raise Exception('Tredje parameter må være en liste med variable.')
+            return
+
+        if not ((coDate==None) | (isinstance(coDate, datetime.datetime))):
+            raise Exception('Fjerde parameter må være en dato.')
+            return
+
+        
+            
 def traverse_hierarchy(keylist, travdf, parqdf, idstreng, hierarchylevels):
     '''
     This function walks through a hierarcical dataset stored in memory,
-    and returns all packed as unpacked data objects.
+    and unpacks all packed data objects.
     
-    The function takes a hirarchical dataset, investigates the schema structre 
+    The function takes a hirarchical dataset, investigates the schema structure 
     to find any other hirarchy elements of type struct, list, and arrays. 
-    All objects found in the data object are subsequently unpacked, 
-    and returend as a data object. 
+    All objects found in these data objects are subsequently unpacked, 
+    and replaces the packed data objects. 
     
     This is a recursive function, which ends when there are no list or array type 
     columns left in the original data object. 
@@ -19,13 +78,13 @@ def traverse_hierarchy(keylist, travdf, parqdf, idstreng, hierarchylevels):
     
     :param keylist: List containing the variables you want to carry forward from the level above, to the level below
     :type keylist: list
-    :param travdf: 
-    :type travdf: 
+    :param travdf: data object to unpack
+    :type travdf: string
     :param parqdf: The hierarchical dataset that you are investigating 
     :type parqdf: dataframe
-    :param idstreng: 
-    :type idstreng: 
-    :param hierarchylevels: controls the depth at which the funcrion will look for arrays 
+    :param idstreng: identifying record in parent dataframe
+    :type idstreng: string
+    :param hierarchylevels: controls the depth at which the function will look for arrays 
     :type hierarchylevels: numeric value
     
      Returns: 
@@ -99,20 +158,22 @@ def traverse_hierarchy(keylist, travdf, parqdf, idstreng, hierarchylevels):
 def unpack_parquet(parqdf, rootdf=False, rootvar=True, levels=-1):
     
     '''
-    This function unpacks a hierarchical parquet dataframe.
+    This function unpacks a hierarchical spark dataframe and relies on function traverse_hierarchy to traverse the hierarchy and unpack.
+    Each unpacked object is a dataframe that gets stored in a dictionary which is then returned to the user.
     
     The parameter rootdf is used to decide whether to create a separate root level dataset without the variables to 
     be extracted. 
         Usage: 
         True: a root level dataset is made. 
         False: a root level dataset is not made. 
-        List: a dataset with root level variables defined in a list. The default value is False.
+        List: a dataset with root level variables defined in a list. 
+        The default value is False.
 
     The parameter rootvar is used to tell how the variables at the root level are to be treated.
         Usage: 
-        True: all variables, which do not need to be unpacked, at the root level, will also be included in all data sets that are unpacked.
-        False: root-level variables will only be included in the root dataset if rootdf is set to True.
-        List: List of variables to include from root level to the extracted datasets
+        True: all variables, which do not need to be unpacked, at the root level, will also be included in all datasets that are unpacked.
+        False: root level variables will not be included in the datasets that are unpacked.
+        List: list of variables from root level to be added to the datasets that are unpacked. 
     
     The levels parameter tells the function how many hierarchy levels it will traverse and unpack. 
         Usage:
@@ -121,13 +182,13 @@ def unpack_parquet(parqdf, rootdf=False, rootvar=True, levels=-1):
         1: the function unpacks one level  
         2: the function unpacks two levels etc.
     
-    :param parqdf: The hierarchical Parquet dataframe to be unpacked.
+    :param parqdf: The dataframe with hierarchical structure to be unpacked.
     :type parqdf: dataframe 
-    :param rootdf: Decides whether to create a separate root level dataset without the variables to be extracted.
-    :type rootdf: Boolean/list 
-    :rootvar: tells how the variables at the root level are to be treated.
-    :type rootvar: Boolean/list
-    :param levels: tells the function how many hierarchy levels the dataset consists of
+    :param rootdf: Decides whether to create a separate root level dataset.
+    :type rootdf: boolean/list 
+    :rootvar: Tells if the root level variables are to be transfered to the unpacked objects or not.
+    :type rootvar: boolean/list
+    :param levels: Tells the function how many hierarchy levels to be traversed
     :type levels: numerical value
     
      Returns: a dictionary
@@ -178,33 +239,3 @@ def unpack_parquet(parqdf, rootdf=False, rootvar=True, levels=-1):
             
     return ds_dict.copy()
         
-def cross_sectional(df, event_var, event_id, coDate=None):
-    '''
-    This function makes a cross sectional dataset of the last record before a defined date
-    for all uniquely identified units in a dataframe (df).
-    The date for which the cross sectional dataset is made is defined in the parameter coDate; 
-    if noe date is given, the default date is today.
-    The records unique id is defined in the parameter event_id. 
-    The time at which the event happens is given in the variable event_var.
-    
-    :param df: The dataframe of observations from which to make a cross sectional dataset 
-    :type df: dataframe 
-    :param event_var: a variable containing the date at which the event happened
-    :type event_var:  time/date 
-    :param event_id: the variables that uniquely define a record
-    :type event_id: list
-    :param coDate: the date at which the cross sectional dataset is taken.
-    :type coDate: time/date 
-    
-    Returns:a dataframe
-    Dataframe: A cross sectional dataframe.
-    '''
-    if coDate!=None:
-        df = df.filter(F.col(event_var) <= coDate)
-    
-    eventlist_ids = event_id.copy()
-    eventlist_ids.append(event_var)
-    
-    df_cross = df.join(df.groupBy(event_id).agg(F.max(event_var).alias(event_var)),\
-                          eventlist_ids, how='inner')
-    return df_cross
\ No newline at end of file
